﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using System.Windows.Input;
using Windows.ApplicationModel;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI.Popups;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;
using FaceIdentify.Models;
using FaceIdentify.Services;
using FaceIdentify.Views;
using Microsoft.ProjectOxford.Face.Contract;
using Template10.Mvvm;

namespace FaceIdentify.ViewModels
{
    public class GroupPageViewModel : ViewModelBase
    {
        private PersonModel _currentPerson;
        private string _personName;
        private IList<Person> _persons;

        public GroupPageViewModel()
        {
            if (DesignMode.DesignModeEnabled)
            {
                //디자인 타임 데이터
            }
            else
            {
                //런타임 데이터
                Init();
            }
        }

        /// <summary>
        ///     현재 작업 중인(추가한) 사용자
        /// </summary>
        public PersonModel CurrentPerson
        {
            get { return _currentPerson; }
            set { Set(ref _currentPerson, value); }
        }

        /// <summary>
        ///     등록된 사용자 목록
        /// </summary>
        public IList<Person> Persons
        {
            get { return _persons; }
            set { Set(ref _persons, value); }
        }

        /// <summary>
        ///     콤보박스 사용자가 선택되는 경우 실행되는 커맨드
        /// </summary>
        public ICommand SelectionChangedPersonCommand { get; set; }

        /// <summary>
        ///     트레이닝 커맨드
        /// </summary>
        public ICommand TrainCommand { get; set; }

        /// <summary>
        ///     사람 제거 커맨드
        /// </summary>
        public ICommand RemovePersonCommand { get; set; }

        /// <summary>
        ///     사람 추가
        /// </summary>
        public ICommand AddPersonCommand { get; set; }

        /// <summary>
        ///     사진 추가
        /// </summary>
        public ICommand AddPictureCommand { get; set; }

        /// <summary>
        ///     사람 이름
        /// </summary>
        public string PersonName
        {
            get { return _personName; }
            set { Set(ref _personName, value); }
        }

        /// <summary>
        ///     초기화
        /// </summary>
        private void Init()
        {
            //커맨드 생성
            AddPersonCommand = new DelegateCommand(AddPersonCommandHandler,
                () => !string.IsNullOrEmpty(PersonName));

            AddPictureCommand = new DelegateCommand(AddPictureCommandHandler);

            TrainCommand = new DelegateCommand(async () =>
            {
                //화면에 Busy 표시
                Busy.SetBusy(true, "Training....");
                await FaceAPIHelper.Instance.TrainPersonGroupAsync();
                Busy.SetBusy(false);
            });

            SelectionChangedPersonCommand = new DelegateCommand<object>(obj =>
            {
                //obj로 넘어온 데이터를 이용해서 현재 사람 표시
                var person = obj as Person;
                if (person == null) return;
                CurrentPerson = new PersonModel
                {
                    Name = person.Name,
                    PersonId = person.PersonId,
                    PersistedFaceIds = person.PersistedFaceIds
                };
            });

            RemovePersonCommand = new DelegateCommand(async () =>
            {
                if (CurrentPerson == null) return;

                Busy.SetBusy(true, "Removing...");
                await FaceAPIHelper.Instance.DeletePersonAsync(CurrentPerson.PersonId);

                Persons = await FaceAPIHelper.Instance.GetPersonsAsync();
                CurrentPerson = null;
                Busy.SetBusy(false);
            });

            PropertyChanged += (s, e) =>
            {
                switch (e.PropertyName)
                {
                    case "PersonName":
                        //커맨드 사용 가능 여부 확인
                        ((DelegateCommand) AddPersonCommand).RaiseCanExecuteChanged();
                        break;
                }
            };
        }

        private async void AddPictureCommandHandler()
        {
            //FileOpePicker 설정
            var openDlg = new FileOpenPicker();
            openDlg.FileTypeFilter.Add(".jpg");
            openDlg.FileTypeFilter.Add(".jpeg");
            openDlg.FileTypeFilter.Add(".png");
            //파일 선택 창 출력
            var file = await openDlg.PickSingleFileAsync();
            var basic = await file.GetBasicPropertiesAsync();
            //서버에 올릴 수 있는 이미지의 크기는 4MB까지입니다.
            if (basic.Size > 4*1024*1024)
            {
                var msg = new MessageDialog("It is too big image. It can only be up to 4MB.");
                await msg.ShowAsync();
                return;
            }
            Busy.SetBusy(true);
            //처음 화면에 출력용
            var bm = new BitmapImage();
            using (var stream = await file.OpenAsync(FileAccessMode.Read))
            {
                await bm.SetSourceAsync(stream);
            }
            //얼굴인식하고 테두리 표시하기
            using (var stream = await file.OpenStreamForReadAsync())
            {
                //얼굴 인식에 실패하는 경우 null을 반환합니다.
                var faceResult = await FaceAPIHelper.Instance.AddPersonFaceAsync(CurrentPerson.PersonId, stream);
                if (faceResult == null) goto ExitRtn;

                CurrentPerson.AddFace(faceResult.PersistedFaceId, bm);
            }
            ExitRtn:
            Busy.SetBusy(false);
        }


        /// <summary>
        ///     사람 추가하기
        /// </summary>
        private async void AddPersonCommandHandler()
        {
            if (string.IsNullOrEmpty(PersonName)) return;
            var personId = await FaceAPIHelper.Instance.CreatePersonAsync(PersonName);

            CurrentPerson = new PersonModel
            {
                Name = PersonName,
                PersonId = personId
            };
        }

        /// <summary>
        ///     Template 10의 기능으로 페이지 네비게이션을 통해서 페이지로 진입하면 실행되는 부분입니다.
        /// </summary>
        /// <param name="parameter"></param>
        /// <param name="mode"></param>
        /// <param name="state"></param>
        /// <returns></returns>
        public override async Task OnNavigatedToAsync(object parameter, NavigationMode mode,
            IDictionary<string, object> state)
        {
            Persons = await FaceAPIHelper.Instance.GetPersonsAsync();
        }
    }
}